/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef AES128_H
#define AES128_H

#include "common.h"
#ifdef ARC_x86
	#ifdef _WIN32
		#include <wmmintrin.h> // AES-NI
	//	#include <immintrin.h> // AVX and older
	//	#include <ammintrin.h> // XOP
	#else
		#include <x86intrin.h>
	#endif
#endif

const uint32_t AES128_KEY_LENGTH = 128 / 32;              //  4
const uint32_t AES128_ROUNDS     = AES128_KEY_LENGTH + 6; // 10
const uint32_t AES128_CTX_SIZE   = 4 * (AES128_ROUNDS + 1);

void newAes128Encrypt(const uint32_t *key, uint32_t *ctx);
void newAes128Decrypt(const uint32_t *key, uint32_t *ctx);
void encryptAes128   (const uint32_t *ctx, uint32_t *inOut);
void decryptAes128   (const uint32_t *ctx, uint32_t *inOut);
#ifdef ARC_x86
	void newAes128Encrypt_NI(const __m128i  *key, __m128i  *ctx);
	void newAes128Decrypt_NI(const __m128i  *key, __m128i  *ctx);
	void encryptAes128_NI   (const __m128i  *ctx, __m128i  *inOut);
	void decryptAes128_NI   (const __m128i  *ctx, __m128i  *inOut);
#endif

#endif
