/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "args.h"
#include <map>
#include <string>

using namespace std;

int searchArg(string &arg, char *value, map<string,char**> &argValues, map<string,int*> &argSwitches);

/**
 * return 0 all good
 * return 1 value used
 * return -1 error
 */
int searchArg(string &arg, char *value, map<string,char**> &argValues, map<string,int*> &argSwitches)
{
	map<string,char**>::iterator itValue;
	map<string,int*  >::iterator itSwitch;
	string argName;
	size_t equal;
	int ret = 0;

	equal = arg.find('=');
	argName = arg.substr(0, equal);
	itValue = argValues.find(argName);
	if (itValue != argValues.end())
	{
		if (equal != string::npos)
		{
			*(itValue->second) = value;
		}
		else if (value != NULL)
		{
			*(itValue->second) = value;
			ret = 1;
		}
		else
		{
			reportError("'-%s' requires a value", arg.c_str());
			ret = -1;
		}
	}
	else
	{
		itSwitch = argSwitches.find(argName);
		if (itSwitch != argSwitches.end())
		{
			if (equal == string::npos)
			{
				*(itSwitch->second) = 1;
			}
			else
			{
				reportError("'-%s' does not have a value", argName.c_str());
				ret = -1;
			}
		}
		else
		{
			reportError("Unknown argument '-%s'", argName.c_str());
			ret = -1;
		}
	}
	return ret;
}

int parseArgs(int argc, char *argv[], programArgs *args)
{
	map<string,char**> argValues;
	map<string,int*  > argSwitches;
	string arg;
	char *value;
	size_t equal;
	int a, b, ret = 0, tmp;

	args->help          = 0;
	args->hash          = NULL;
	args->hashFile      = NULL;
	args->input         = NULL;
	args->inputPrecomp  = NULL;
	args->output        = NULL;
	args->outputCracked = NULL;
	args->threads      = NULL;

	argSwitches["-help"]         = &(args->help);
	argValues["-hash"]           = &(args->hash);
	argValues["h"]               = &(args->hash);
	argValues["-hash-file"]      = &(args->hashFile);
	argValues["H"]               = &(args->hashFile);
	argValues["-input"]          = &(args->input);
	argValues["i"]               = &(args->input);
	argValues["-input-precomp"]  = &(args->inputPrecomp);
	argValues["I"]               = &(args->inputPrecomp);
	argValues["-output"]         = &(args->output);
	argValues["o"]               = &(args->output);
	argValues["-output-cracked"] = &(args->outputCracked);
	argValues["O"]               = &(args->outputCracked);
	argValues["-threads"]        = &(args->threads);
	argValues["t"]               = &(args->threads);

	for (a = 1; a < argc; a++)
	{
		if (argv[a][0] == '-')
		{
			value = NULL;
			if (argv[a][1] == '-')
			{
				arg = argv[a] + 1;
				equal = arg.find('=');
				if (equal != string::npos)
				{
					value = argv[a] + (equal + 1);
				}
				else if (a + 1 < argc)
				{
					value = argv[a + 1];
				}
				switch (searchArg(arg, value, argValues, argSwitches))
				{
					case  1: a++; break;
					case -1: ret = 1;
				}
			}
			else
			{
				for (b = 1; argv[a][b] != 0 && argv[a][b] != '='; b++)
				{
					arg = argv[a][b];
					if (argv[a][b + 1] == '=')
					{
						arg = argv[a] + b;
						value = argv[a] + (b + 2);
					}
					else if (argv[a][b + 1] == 0 && a + 1 < argc)
					{
						value = argv[a + 1];
					}
					tmp = searchArg(arg, value, argValues, argSwitches);
					if (tmp == 1)
					{
						a++;
						break;
					}
					else if (tmp != 0)
					{
						ret = 1;
						break;
					}
				}
			}
		}
		else
		{
			reportError("Unknown argument '%s'", argv[a]);
			ret = 1;
		}
	}
	return ret;
}
