/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "common.h"

uint32_t getInstructionSets()
{
#ifdef ARC_x86
	static uint32_t ret = 0xffffffff;

	if (ret == 0xffffffff)
	{
		uint32_t out1, out2, out3, out4, out5;

#ifdef _WIN32
		__asm
		{
			mov   eax,0x80000001
			cpuid
			mov   eax,1
			mov   out3,ecx
			mov   out4,edx
			cpuid
			mov   out1,ecx
			mov   out2,edx
		}
#else
		asm(
			"movl   $2147483649,%%eax\n\t"
			"cpuid\n\t"
			"movl   $1,%%eax\n\t"
			"movl   %%ecx,%2\n\t"
			"movl   %%edx,%3\n\t"
			"cpuid"
			: "=c"(out1), "=d"(out2), "=m"(out3), "=m"(out4) // output
			: // input
			: "eax", "ebx"); // used
#endif
		ret = 0;
		if (out2 & (1 << 23))
		{
			ret |= IS_MMX;
		}
		if (out2 & (1 << 25))
		{
			ret |= IS_SSE;
		}
		if (out2 & (1 << 26))
		{
			ret |= IS_SSE2;
		}
		if (out1 & (1 << 0))
		{
			ret |= IS_SSE3;
		}
		if (out1 & (1 << 9))
		{
			ret |= IS_SSSE3;
		}
		if (out1 & (1 << 19))
		{
			ret |= IS_SSE41;
		}
		if (out1 & (1 << 20))
		{
			ret |= IS_SSE42;
		}
		if (out3 & (1 << 6))
		{
			ret |= IS_SSE4A;
		}
		if (out1 & (1 << 28))
		{
			ret |= IS_AVX;
		}
		if ((ret & IS_AVX) && (out1 & (1 << 27))) // OSXSAVE (XGETBV)
		{
#ifdef _WIN32
			__asm
			{
				xor   ecx,ecx
				xgetbv
				mov   out5,eax
			}
#else
			asm(
				"xor   %%ecx,%%ecx\n\t"
				"xgetbv"
				: "=a"(out5) // output
				: // input
				: "ecx", "edx"); // used
#endif
			if ((out5 & 0x6) == 0x6)
			{
				ret |= IS_OS_YMM;
			}
		}
		if (ret & IS_OS_YMM)
		{
#ifdef _WIN32
			__asm
			{
				mov   eax,7
				xor   ecx,ecx
				cpuid
				mov   out5,ebx
			}
#else
			asm(
				"movl   $7,%%eax\n\t"
				"xor   %%ecx,%%ecx\n\t"
				"cpuid\n\t"
				: "=b"(out5) // output
				: // input
				: "eax", "ecx", "edx"); // used
#endif
			if (out5 & (1 << 5))
			{
				ret |= IS_AVX2;
			}
		}
		if (out3 & (1 << 11))
		{
			ret |= IS_XOP;
		}
		if (out1 & (1 << 23))
		{
			ret |= IS_POPCNT;
		}
		if (out1 & (1 << 25))
		{
			ret |= IS_AESNI;
		}
#ifdef ARC_x86_32
#ifndef _WIN32 // Remove if you compiled both 64 bit and 32 bit versions for Windows
		if ((out2 & (1 << 30)) || (out4 & (1 << 29)) || (out3 & (1 << 6))) // x86_64, long_mode, long_mode_intel
		{
			reportWarning("Your CPU can run 64 bit which runs faster than the 32 bit code you are currently running.");
		}
#endif
#endif
	}
	return ret;
#else
	return 0;
#endif
}

void reportError(const char *msg, ...)
{
	va_list args;
	char *format;
	size_t len = strlen(msg);

	format = new char[len + 9]; // 9 == sizeof("Error: \n")
	strcpy(format, "Error: ");
	strcpy(format + 7, msg);
	format[len + 7] = '\n';
	format[len + 8] = 0;
	va_start(args, msg);
	vfprintf(stderr, format, args);
	va_end(args);
	delete [] format;
}

void reportWarning(const char *msg, ...)
{
	va_list args;
	char *format;
	size_t len = strlen(msg);

	format = new char[len + 11]; // 11 == sizeof("Warning: \n")
	strcpy(format, "Warning: ");
	strcpy(format + 9, msg);
	format[len + 9] = '\n';
	format[len + 10] = 0;
	va_start(args, msg);
	vfprintf(stderr, format, args);
	va_end(args);
	delete [] format;
}
