/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef COMMON_H
#define COMMON_H

#include "architecture.h"
#include <stdio.h>
#include <assert.h>

#ifdef _WIN32
	#pragma warning(disable:4996)
#endif

#ifndef __inline__
	#if defined(_MSC_VER) || defined(__INTEL_COMPILER)
		#define __inline__  __forceinline
	#elif defined(__GNUC__)
		#define __inline__  __inline __attribute__((__always_inline__))
	#else
		#define __inline__  inline
	#endif
#endif

#define __STDC_CONSTANT_MACROS
#define __STDC_FORMAT_MACROS
#define __STDC_LIMIT_MACROS
#ifdef _WIN32
	#include "inttypes.h"
	#include <time.h>
	#include <windows.h>

	#include <string.h>
	#define STR_TO_ULL(str,endPtr,radix) _strtoui64(str, endPtr, radix)
	#define STR_CMP_ICASE(a,b) strcmpi(a, b)

	typedef LARGE_INTEGER TIMER_TYPE;
	#define TIMER_FUNC(t)             QueryPerformanceCounter(&t)
	__inline__ double TIMER_DIFF(LARGE_INTEGER s, LARGE_INTEGER e)
	{
		LARGE_INTEGER f;
		QueryPerformanceFrequency(&f);
		return ((double) (e.QuadPart - s.QuadPart)) / f.QuadPart;
	}
	#define DIRECTORY_SEPERATOR       "\\"
	#define DIRECTORY_SEPERATOR_CHAR  '\\'

	#define WIN_ALIGN_16 __declspec(align(16))
	#define ALIGN_16
	#define FSEEK64(file,off,origin)  _fseeki64(file,off,origin)
	#define GET_EXE_FULL_NAME(buf,bufSize)  GetModuleFileNameA(NULL, buf, bufSize)
	#define SLEEP_MS(ms) Sleep(ms)
#else
	#include <inttypes.h>
	#include <sys/time.h>
	#include <unistd.h>
	#include <time.h>
	//#include <fcntl.h>

	#include <strings.h>
	#define STR_TO_ULL(str,endPtr,radix) strtoull(str, endPtr, radix)
	#define STR_CMP_ICASE(a,b) strcasecmp(a, b)

	typedef timeval TIMER_TYPE;
	#define TIMER_FUNC(t)             gettimeofday(&t, NULL)
	#define TIMER_DIFF(s,e)           ((e.tv_sec - s.tv_sec) + (e.tv_usec - s.tv_usec) / (double)1000000.0)
	#define DIRECTORY_SEPERATOR       "/"
	#define DIRECTORY_SEPERATOR_CHAR  '/'

	#ifndef PRIu64
		#define PRIu64 "llu"
	#endif
	#ifndef PRIx64
		#define PRIx64 "llx"
	#endif
	#ifndef UINT64_MAX
		#define UINT64_MAX 0xffffffffffffffffull
	#endif
	#ifndef UINT32_MAX
		#define UINT32_MAX 0xffffffff
	#endif

	#define WIN_ALIGN_16
	#define ALIGN_16 __attribute__((aligned(16)))
	#define FSEEK64(file,off,origin)  fseeko64(file,off,origin)
	#define GET_EXE_FULL_NAME(buf,bufSize)  readlink("/proc/self/exe", buf, bufSize);
	__inline__ void SLEEP_MS(uint32_t ms)
	{
		timespec time;

		time.tv_sec  = ms / 1000;
		time.tv_nsec = (ms % 1000) * 1000000;
		nanosleep(&time, NULL);
	}
#endif

const uint32_t PW_BUFFER_SIZE      = 1024;
const uint32_t MAX_PW_BLOCKS       = PW_BUFFER_SIZE / 16;
const uint32_t NODE_MAX_PWS        = 256;
const uint32_t NODE_PW_BUFFER_SIZE = 16 * NODE_MAX_PWS;

#define SWAP_8_LO_BITS_BIT_ENDIAN_24(x)  ((((x) & 0x01) << 23) | (((x) & 0x02) << 22) | (((x) & 0x04) << 21) | (((x) & 0x08) << 20) | \
                                          (((x) & 0x10) << 19) | (((x) & 0x20) << 18) | (((x) & 0x40) << 17) | (((x) & 0x80) << 16))
#define SWAP_8_LO_BITS_BIT_ENDIAN_32(x)  ((((x) & 0x01) << 31) | (((x) & 0x02) << 30) | (((x) & 0x04) << 29) | (((x) & 0x08) << 28) | \
                                          (((x) & 0x10) << 27) | (((x) & 0x20) << 26) | (((x) & 0x40) << 25) | (((x) & 0x80) << 24))
#define SWAP_ENDIAN_32(x)                (((x) >> 24) | ((x) << 24) | (((x) >> 8) & 0xff00) | (((x) << 8) & 0xff0000))
#define SWAP_ENDIAN_16(x)                (((x) >> 8) | ((x) << 8))

#ifdef ARC_LITTLE_ENDIAN
	#define TO_BIG_ENDIAN_32(x)     SWAP_ENDIAN_32(x)
	#define TO_BIG_ENDIAN_16(x)     SWAP_ENDIAN_16(x)
	#define TO_LITTLE_ENDIAN_32(x)  (x)
	#define TO_LITTLE_ENDIAN_16(x)  (x)
#else
	#define TO_BIG_ENDIAN_32(x)     (x)
	#define TO_BIG_ENDIAN_16(x)     (x)
	#define TO_LITTLE_ENDIAN_32(x)  SWAP_ENDIAN_32(x)
	#define TO_LITTLE_ENDIAN_16(x)  SWAP_ENDIAN_16(x)
#endif
#define FROM_BIG_ENDIAN_32(x)     TO_BIG_ENDIAN_32(x)
#define FROM_BIG_ENDIAN_16(x)     TO_BIG_ENDIAN_16(x)
#define FROM_LITTLE_ENDIAN_32(x)  TO_LITTLE_ENDIAN_32(x)
#define FROM_LITTLE_ENDIAN_16(x)  TO_LITTLE_ENDIAN_16(x)

union uint32u
{
	uint32_t u32;
	uint16_t u16[2];
	uint8_t  u8[4];
};

enum instructionSets
{
	IS_MMX     = 0x0001,
	IS_SSE     = 0x0002,
	IS_SSE2    = 0x0004,
	IS_SSE3    = 0x0008,
	IS_SSSE3   = 0x0010, // pshufb xmmd,xmms
	IS_SSE41   = 0x0020, // ptest xmm1,xmm2
	IS_SSE42   = 0x0040,
	IS_SSE4A   = 0x0080,
	IS_AVX     = 0x0100,
	IS_AVX2    = 0x0200,
	IS_OS_YMM  = 0x0400,
	IS_XOP     = 0x0800, // vprotd xmmd,xmms,imm8; vpcmov xmmd,xmms1,xmms2,xmmc
	IS_POPCNT  = 0x1000, // popcnt r,r/m16; popcnt r,r/m32; popcnt r,r/m64
	IS_TZCNT   = 0x2000,  // tzcnt  r,r/m16; tzcnt  r,r/m32; tzcnt  r,r/m64
	IS_AESNI   = 0x4000
};

uint32_t getInstructionSets();
void reportError(const char *msg, ...);
void reportWarning(const char *msg, ...);

#endif
