/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include <list>
#include <string>
#include <string.h>
#include "common.h"
#include "aes128.h"
#include "args.h"
#include "messagequeue.h"
#include "msgqueuenodetype.h"
#include "megahashgen.h"
#include "megahashgenni.h"
#include "megahashchecker.h"
#include "megahashcheckerni.h"
#include "reader.h"

using namespace std;
/*
void megaPw2ArrU32(const uint32_t *pwIn, uint32_t *pwOut, uint32_t pwLen)
{
	for (uint32_t i = 0; i < pwLen; i++)
	{
		if ((i & 3) == 0)
		{
			pwOut[i >> 2] = 0;
		}
		pwOut[i >> 2] |= (pwIn[i] << (24 - (i & 3) * 8)); // [sic]
	}
}

void megaPw2ArrU(const uint16_t *pwIn, uint32_t *pwOut, uint32_t pwLen)
{
	for (uint32_t i = 0; i < pwLen; i++)
	{
		if ((i & 3) == 0)
		{
			pwOut[i >> 2] = 0;
		}
		pwOut[i >> 2] |= (pwIn[i] << (24 - (i & 3) * 8)); // [sic]
	}
}

void megaPw2ArrA(const uint8_t *pwIn, uint32_t *pwOut, uint32_t pwLen)
{
	for (uint32_t i = 0; i < pwLen; i++)
	{
		if ((i & 3) == 0)
		{
			pwOut[i >> 2] = 0;
		}
		pwOut[i >> 2] |= (pwIn[i] << (24 - (i & 3) * 8));
	}
	for (uint32_t i = pwLen + 3; i / 16 < (pwLen + 15) / 16; i += 4)
	{
		pwOut[i >> 2] = 0;
	}
}
*/

// in = "[[[https://]mega.co.nz/]#confirm].*"
int megaGetHash(const char *in, uint32_t *outHash)
{
	const int b64ToDec[] =
	{
		// ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_
		62, -1, -1, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, 63, -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51
	};

	if (memcmp(in, "https://mega.co.nz/#confirm", sizeof("https://mega.co.nz/#confirm") - 1) == 0)
	{
		in += sizeof("https://mega.co.nz/#confirm") - 1;
	}
	else if (memcmp(in, "mega.co.nz/#confirm", sizeof("mega.co.nz/#confirm") - 1) == 0)
	{
		in += sizeof("mega.co.nz/#confirm") - 1;
	}
	else if (memcmp(in, "#confirm", sizeof("#confirm") - 1) == 0)
	{
		in += sizeof("#confirm") - 1;
	}
	if (strlen(in) < 43)
	{
		return 1;
	}

	// Skip 128 bits
	in += 21;

	// Get 128 bits
	// Ignore first 2 bits
	int ch;
	uint32_t data;
	uint32_t bits = 4;
	if (in[0] < 45 || in[0] > 122 || (ch = b64ToDec[in[0] - 45]) == -1)
	{
		return 1;
	}
	data = ch;
	for (uint32_t i = 1, j = 0; i < 22; i++)
	{
		if (in[i] < 45 || in[i] > 122 || (ch = b64ToDec[in[i] - 45]) == -1)
		{
			return 1;
		}
		if (bits + 6 >= 32)
		{
			outHash[j++] = (data << (32 - bits)) | (((uint32_t) ch) >> (6 - 32 + bits));
			bits -= 32;
		}
		bits += 6;
		data = (data << 6) | (uint32_t) ch;
	}
	return 0;
}

int precompAttack(FILE *fin, list<pair<uint32_t*,string> > &hashes)
{
	char pw[PW_BUFFER_SIZE];
#ifdef ARC_x86
	__m128i hashCtx[AES128_CTX_SIZE / 4];
	__m128i hash[1];
#else
	uint32_t hashCtx[AES128_CTX_SIZE];
	uint32_t hash[4];
#endif
	uint64_t count = 0;
	TIMER_TYPE s, e;
	int ret = 0;
	bool useAesNi = (getInstructionSets() & IS_AESNI) != 0;

	TIMER_FUNC(s);
	while (1)
	{
		if (fgets(pw, PW_BUFFER_SIZE, fin) == NULL)
		{
			if (!feof(fin))
			{
				ret = 1;
				perror("fgets");
			}
			break;
		}
		if (fread(hash, 16, 1, fin) != 1)
		{
			ret = 1;
			perror("fread");
			break;
		}

		// Test password
#ifdef ARC_x86
		if (useAesNi)
		{
			newAes128Decrypt_NI(hash, hashCtx);
		}
		else
		{
			newAes128Decrypt((uint32_t*) hash, (uint32_t*) hashCtx);
		}
#else
		newAes128Decrypt((uint32_t*) hash, (uint32_t*) hashCtx);
#endif
		count++;

		list<pair<uint32_t*,string> >::iterator itDel = hashes.end();
		for (list<pair<uint32_t*,string> >::iterator it = hashes.begin() ; it != hashes.end(); ++it)
		{
#ifdef ARC_x86
			__m128i tmp[1];
#else
			uint32_t tmp[4];
#endif

			// Remove found
			if (itDel != hashes.end())
			{
				hashes.erase(itDel);
				itDel = hashes.end();
			}

			// Check
			memcpy(tmp, it->first, 16);
#ifdef ARC_x86
			if (useAesNi)
			{
				decryptAes128_NI(hashCtx, tmp);
			}
			else
			{
				decryptAes128((uint32_t*) hashCtx, (uint32_t*) tmp);
			}
#else
			decryptAes128((uint32_t*) hashCtx, (uint32_t*) tmp);
#endif
			if (((uint32_t*) tmp)[1] == 0 && ((uint32_t*) tmp)[2] == 0)
			{
				// Output cracked password
				printf("%s:%s", it->second.c_str(), pw); // Note pw has a trailing '\n'
				itDel = it;
			}
		}

		// Remove found
		if (itDel != hashes.end())
		{
			hashes.erase(itDel);
			if (hashes.size() == 0)
			{
				break;
			}
		}
	}
	TIMER_FUNC(e);
	printf("Checked %" PRIu64 " passwords in %0.3f sec (%0.3f c/s)\n", count, TIMER_DIFF(s, e), count / TIMER_DIFF(s, e));
	return ret;
}

void usage(const char *runStr)
{
	printf("\n"
		"                          **** MegaCracker v0.2a ****\n\n"
		"Cracks a Mega password from a confirmation link. Do NOT post your confirmation\n"
		"link because it contains not only a hash of your password but your encrypted\n"
		"master key. Once the master key is disclosed all your file can be decrypted.\n\n"
		"%s [options]\n\n"
		"  -h|--hash=<string>\n"
		"    Confirmation link.\n\n"
		"  -H|--hash-file=<file>\n"
		"    File with confirmation links.\n\n"
		"  --help\n"
		"    Display help and exit.\n\n"
		"  -i|--input=<file>\n"
		"    File with passwords to test [default is stdin].\n\n"
		"  -I|--input-precomp=<file>\n"
		"    File with pregenerated data.\n\n"
		"  -o|--output=<file>\n"
		"    Output data to use later.\n\n"
		"  -O|--output-cracked=<file>\n"
		"    Output cracked passwords to file [default is stdout].\n\n"
		"  -t|--threads=<int>\n"
		"    Number of CPU threads [default is number of cores].\n\n\n"
		"Copyright (c) 2013 Steve Thomas <steve AT tobtu DOT com>\n"
		"  This is free software: you can redistribute it and/or modify it under the\n"
		"  terms of the GNU General Public License as published by the Free Software\n"
		"  Foundation, either version 2 of the License, or (at your option) any later\n"
		"  version. There is NO WARRANTY, to the extent permitted by law.\n", runStr);
}

int main(int argc, char *argv[])
{
	programArgs programArgs;
	list<pair<uint32_t*,string> > hashes;
	int ret;
	bool useAesNi = (getInstructionSets() & IS_AESNI) != 0;

	// Init args
	if (argc == 1)
	{
		usage(argv[0]);
		return 1;
	}
	ret = parseArgs(argc, argv, &programArgs);
	if (programArgs.help)
	{
		usage(argv[0]);
		return ret;
	}

	if (programArgs.hash == NULL && programArgs.hashFile == NULL && programArgs.output == NULL)
	{
		reportError("Missing argument -h|--hash, -H|--hash-file, or -o|--output\n");
		ret = 1;
	}
	if (programArgs.input != NULL && programArgs.inputPrecomp != NULL)
	{
		reportError("Can't have both -i|--input and -I|--input-precomp\n");
		ret = 1;
	}
	if (programArgs.inputPrecomp != NULL && programArgs.output != NULL)
	{
		reportError("Can't have both -I|--input-precomp and -o|--output\n");
		ret = 1;
	}
	if (ret)
	{
		usage(argv[0]);
		return ret;
	}

	// Set number of threads
	uint32_t numThreads = getNumCores();
	if (programArgs.threads)
	{
		numThreads = atoi(programArgs.threads);
		if (numThreads > 128)
		{
			reportError("Max number of threads is 128... are you sure you know what you are doing?");
			return 1;
		}
		if (numThreads == 0)
		{
			reportError("Need at least 1 thread.");
			return 1;
		}
	}

	// Get hashes
	if (programArgs.hash != NULL)
	{
		uint32_t *hash = new uint32_t[4];

		if (megaGetHash(programArgs.hash, hash))
		{
			reportError("Invalid hash \"%s\"\n", programArgs.hash);
			ret = 1;
		}
		else
		{
			if (useAesNi)
			{
				hash[0] = SWAP_ENDIAN_32(hash[0]);
				hash[1] = SWAP_ENDIAN_32(hash[1]);
				hash[2] = SWAP_ENDIAN_32(hash[2]);
				hash[3] = SWAP_ENDIAN_32(hash[3]);
			}
			hashes.push_back(make_pair<uint32_t*,string>(hash, programArgs.hash));
		}
	}
	if (programArgs.hashFile != NULL)
	{
		FILE *fin = fopen(programArgs.hashFile, "r");

		if (fin == NULL)
		{
			perror("fopen");
			ret = 1;
		}
		else
		{
			while (1)
			{
				char str[PW_BUFFER_SIZE];

				if (fgets(str, PW_BUFFER_SIZE, fin) == NULL)
				{
					if (!feof(fin))
					{
						ret = 1;
						perror("fgets");
					}
					break;
				}
				uint32_t *hash = new uint32_t[4];
				if (megaGetHash(str, hash))
				{
					reportError("Invalid hash \"%s\"\n", programArgs.hash);
					ret = 1;
				}
				else
				{
					if (useAesNi)
					{
						hash[0] = SWAP_ENDIAN_32(hash[0]);
						hash[1] = SWAP_ENDIAN_32(hash[1]);
						hash[2] = SWAP_ENDIAN_32(hash[2]);
						hash[3] = SWAP_ENDIAN_32(hash[3]);
					}
					size_t pwLen = strlen(str);
					while (pwLen > 0 && (str[pwLen - 1] == '\r' || str[pwLen - 1] == '\n'))
					{
						pwLen--;
					}
					str[pwLen] = 0;
					hashes.push_back(make_pair<uint32_t*,string>(hash, str));
				}
			}
		}
	}
	if (ret)
	{
		return ret;
	}
	if (hashes.size() == 0 && programArgs.output == NULL)
	{
		reportError("No hashes and no output\n");
		return 1;
	}
	if (hashes.size() == 0 && programArgs.inputPrecomp != NULL)
	{
		reportError("No hashes\n");
		return 1;
	}

	FILE *fin = stdin;
	FILE *fout = NULL;
	FILE *foutCracked = stdout;

	// Open input
	if (programArgs.input)
	{
		fin = fopen(programArgs.input, "rb");
		if (fin == NULL)
		{
			perror("fopen");
			return 1;
		}
	}
	else if (programArgs.inputPrecomp != NULL)
	{
		fin = fopen(programArgs.inputPrecomp, "rb");
		if (fin == NULL)
		{
			perror("fopen");
			return 1;
		}
		return precompAttack(fin, hashes);
	}

	// Open output
	if (programArgs.output)
	{
		fout = fopen(programArgs.output, "wb");
		if (fout == NULL)
		{
			perror("fopen");
			return 1;
		}
	}

	// Open output cracked
	if (programArgs.outputCracked)
	{
		foutCracked = fopen(programArgs.outputCracked, "a");
		if (foutCracked == NULL)
		{
			perror("fopen");
			return 1;
		}
	}

	// Output number of threads
	printf("Initializing with %u cracker threads\n", numThreads);

	// Init threads and queues
	MessageQueue<msgQueueNodeType> workUnitsQueue;
	MessageQueue<msgQueueNodeType> workDoneQueue;
	MessageQueue<msgQueueNodeType> freeQueue;
	MegaHashGen **threads = NULL;
	MegaHashChecker *checkerThread = NULL;
	messageQueueNode<msgQueueNodeType> *nodes;

	// .----> Reader -> workUnitsQueue -> MegaHashGen ----.
	// |                                                  |
	// '- freeQueue <- MegaHashChecker <- workDoneQueue <-'

	// Init reader thread
	Reader reader(&freeQueue, &workUnitsQueue, fin);
	reader.start();

	// Init workUnitsQueue/workDoneQueue/freeQueue queues
	nodes = new messageQueueNode<msgQueueNodeType>[4 * numThreads];
	for (uint32_t i = 0; i < 4 * numThreads; i++)
	{
		nodes[i].next = nodes + (i + 1);
		nodes[i].data.pwKeys    = new uint32_t[4 * NODE_MAX_PWS];
		nodes[i].data.passwords = new char[NODE_PW_BUFFER_SIZE];
	}
	nodes[4 * numThreads - 1].next = NULL;
	freeQueue.pushQueue(nodes, nodes + (4 * numThreads - 1));

	// Init threads
#ifdef ARC_x86
	MegaHashGen_NI **threads_NI = NULL;
	MegaHashChecker_NI *checkerThread_NI = NULL;

	if (useAesNi)
	{
		threads_NI = new MegaHashGen_NI*[numThreads];
		for (uint32_t i = 0; i < numThreads; i++)
		{
			threads_NI[i] = new MegaHashGen_NI(&workUnitsQueue, &workDoneQueue);
			threads_NI[i]->start();
		}
		checkerThread_NI = new MegaHashChecker_NI(&workDoneQueue, &freeQueue, hashes, fout, foutCracked);
		checkerThread_NI->start();
	}
	else
	{
#endif
		threads = new MegaHashGen*[numThreads];
		for (uint32_t i = 0; i < numThreads; i++)
		{
			threads[i] = new MegaHashGen(&workUnitsQueue, &workDoneQueue);
			threads[i]->start();
		}
		checkerThread = new MegaHashChecker(&workDoneQueue, &freeQueue, hashes, fout, foutCracked);
		checkerThread->start();
#ifdef ARC_x86
	}
#endif

	// Check for stalled stdin
	if (fin == stdin)
	{
		for (uint32_t ms = 1; ms <= 1000 && !reader.read(); ms *= 10)
		{
			SLEEP_MS(ms);
		}
		if (!reader.read())
		{
			reportWarning("Still waiting for stdin, are you missing -i|--input or -I|--input-precomp?");
		}
	}

	// Wait for threads to finish
	reader.waitForStopRunning();
	for (uint32_t i = 0; i < numThreads; i++)
	{
#ifdef ARC_x86
		if (useAesNi)
		{
			threads_NI[i]->waitForStopRunning();
		}
		else
		{
			threads[i]->waitForStopRunning();
		}
#else
		threads[i]->waitForStopRunning();
#endif
	}
	workDoneQueue.flush();
#ifdef ARC_x86
	if (useAesNi)
	{
		checkerThread_NI->waitForStopRunning();
	}
	else
	{
		checkerThread->waitForStopRunning();
	}
#else
	checkerThread->waitForStopRunning();
#endif

	// Clean up
	for (uint32_t i = 0; i < numThreads; i++)
	{
#ifdef ARC_x86
		if (useAesNi)
		{
			delete threads_NI[i];
		}
		else
		{
			delete threads[i];
		}
#else
		delete threads[i];
#endif
	}
#ifdef ARC_x86
	if (useAesNi)
	{
		delete [] threads_NI;
		delete checkerThread_NI;
	}
	else
	{
		delete [] threads;
		delete checkerThread;
	}
#else
	delete [] threads;
	delete checkerThread;
#endif
	for (uint32_t i = 0; i < 4 * numThreads; i++)
	{
		delete [] nodes[i].data.pwKeys;
		delete [] nodes[i].data.passwords;
	}
	delete [] nodes;
	return 0;
}
