/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdlib.h>
#include <string.h>
#include "megahashcheckerni.h"
#include "aes128.h"

using namespace std;

MegaHashChecker_NI::MegaHashChecker_NI(
	MessageQueue<msgQueueNodeType>        *inQueue,
	MessageQueue<msgQueueNodeType>        *outQueue,
	list<pair<uint32_t*,string> >         &hashes,
	FILE                                  *fout,
	FILE                                  *foutCracked)
	: m_inQueue(inQueue),
	  m_outQueue(outQueue),
	  m_hashes(&hashes),
	  m_fout(fout),
	  m_foutCracked(foutCracked),
	  m_run(1)
{
	if (inQueue == NULL || outQueue == NULL)
	{
		reportError("MegaHashChecker_NI::MegaHashChecker_NI(): NULL pointer");
		exit(1);
	}
}

MegaHashChecker_NI::~MegaHashChecker_NI()
{
	if (!stopRunning())
	{
		waitForStopRunning();
	}
}

int MegaHashChecker_NI::stopRunning()
{
	if (m_run)
	{
		m_run = 0;
		m_inQueue->flush();
		return 0;
	}
	return 1;
}

void *MegaHashChecker_NI::threadFunc()
{
#ifdef ARC_x86
	__m128i pwKeyCtx[AES128_CTX_SIZE / 4];
	__m128i pwKey;
	__m128i hash;
	TIMER_TYPE                          s, e;
	uint64_t                            hashCount   = 0;
	MessageQueue<msgQueueNodeType>     *inQueue     = m_inQueue;
	MessageQueue<msgQueueNodeType>     *outQueue    = m_outQueue;
	messageQueueNode<msgQueueNodeType> *node;
	list<pair<uint32_t*,string> >      *hashes      = m_hashes;
	FILE                               *fout        = m_fout;
	FILE                               *foutCracked = m_foutCracked;
	uint32_t *pwKeys;
	uint8_t  *passwords;

	TIMER_FUNC(s);

	// Get work unit
	node = inQueue->pop();
	while (node != NULL && m_run)
	{
		pwKeys     = node->data.pwKeys;
		passwords  = (uint8_t*) node->data.passwords;
		hashCount += node->data.count;
		for (uint32_t i = 0, count = node->data.count; i < count; i++)
		{
			// Get password
			uint32_t pwLen = *((uint16_t*) passwords);
			passwords += 2;

			// Write precomp data
			if (fout != NULL)
			{
				if (fwrite(passwords, pwLen, 1, fout) != 1 ||
				    fwrite("\n",      1,     1, fout) != 1 ||
				    fwrite(pwKeys,    16,    1, fout) != 1)
				{
					perror("fwrite");
				}
			}

			// Test password
			if (hashes->size() != 0)
			{
				memcpy(&pwKey, pwKeys, 16);
				newAes128Decrypt_NI(&pwKey, pwKeyCtx);
				list<pair<uint32_t*,string> >::iterator itDel = hashes->end();
				for (list<pair<uint32_t*,string> >::iterator it = hashes->begin(); it != hashes->end(); ++it)
				{
					// Remove found
					if (itDel != hashes->end())
					{
						hashes->erase(itDel);
						itDel = hashes->end();
					}

					// Check
					memcpy(&hash, it->first, 16);
					decryptAes128_NI(pwKeyCtx, &hash);
					if (((uint32_t*) &hash)[1] == 0 && ((uint32_t*) &hash)[2] == 0)
					{
						// Output cracked password
						string str = it->second + ':';

						str.append((char*) passwords, pwLen);
						fprintf(foutCracked, "%s\n", str.c_str());
						itDel = it;
					}
				}

				// Remove found
				if (itDel != hashes->end())
				{
					hashes->erase(itDel);
					if (hashes->size() == 0 && fout == NULL)
					{
						// *** cough *** break 2;
						goto EXIT_LOOP;
					}
				}
			}
			passwords += pwLen;
			pwKeys += 4;
		}

		// Get next work unit
		outQueue->push(node);
		node = inQueue->pop();
	}

EXIT_LOOP:
#else
	reportError("MegaHashGen_NI::threadFunc() You are trying to run x86 code on a non-x86 CPU.");
	exit(1);
#endif
	TIMER_FUNC(e);
	printf("Checked %" PRIu64 " passwords in %0.3f sec (%0.3f c/s)\n", hashCount, TIMER_DIFF(s, e), hashCount / TIMER_DIFF(s, e));
	m_run = 0;
	inQueue->flush();
	outQueue->flush();
	return NULL;
}
