/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdlib.h>
#include <string.h>
#include "megahashgen.h"
#include "aes128.h"

MegaHashGen::MegaHashGen(
	MessageQueue<msgQueueNodeType> *inQueue,
	MessageQueue<msgQueueNodeType> *outQueue)
	: m_inQueue(inQueue),
	  m_outQueue(outQueue),
	  m_run(1)
{
	if (inQueue == NULL || outQueue == NULL)
	{
		reportError("MegaHashGen::MegaHashGen(): NULL pointer");
		exit(1);
	}
}

MegaHashGen::~MegaHashGen()
{
	if (!stopRunning())
	{
		waitForStopRunning();
	}
}

int MegaHashGen::stopRunning()
{
	if (m_run)
	{
		m_run = 0;
		m_inQueue->flush();
		return 0;
	}
	return 1;
}

void *MegaHashGen::threadFunc()
{
	uint32_t ctx[AES128_CTX_SIZE * MAX_PW_BLOCKS];
	uint32_t pw[PW_BUFFER_SIZE / sizeof(uint32_t) + 4];
	uint32_t pwKey[4];
	const uint32_t PW_KEY_INIT[4] = {0x93c467e3, 0x7db0c7a4, 0xd1be3f81, 0x0152cb56};
	MessageQueue<msgQueueNodeType> *inQueue  = m_inQueue;
	MessageQueue<msgQueueNodeType> *outQueue = m_outQueue;
	messageQueueNode<msgQueueNodeType> *node;
	uint32_t *pwKeys;
	uint8_t *passwords;

	// Get work unit
	node = inQueue->pop();
	while (node != NULL && m_run)
	{
		pwKeys = node->data.pwKeys;
		passwords = (uint8_t*) node->data.passwords;
		for (uint32_t i = 0, count = node->data.count; i < count; i++)
		{
			// Get password
			uint32_t pwLen = *((uint16_t*) passwords);
			uint32_t pwBlocks = (pwLen + 15) / 16;

			passwords += 2;
			pw[pwLen / 4    ] = 0;
			pw[pwLen / 4 + 1] = 0;
			pw[pwLen / 4 + 2] = 0;
			pw[pwLen / 4 + 3] = 0;
			for (uint32_t j = 0; j < pwLen; j += 4)
			{
				uint32_t tmp = passwords[j] << 24;
				if (j + 1 < pwLen)
				{
					tmp |= passwords[j + 1] << 16;
					if (j + 2 < pwLen)
					{
						tmp |= passwords[j + 2] << 8;
						if (j + 3 < pwLen)
						{
							tmp |= passwords[j + 3];
						}
					}
				}
				pw[j >> 2] = tmp;
			}
			passwords += pwLen;

			// MegaKDF
			memcpy(pwKey, PW_KEY_INIT, sizeof(pwKey));
			if (pwBlocks)
			{
				newAes128Encrypt(pw, ctx);
				for (uint32_t j = 1; j < pwBlocks; j++)
				{
					newAes128Encrypt(pw + 4 * j, ctx + AES128_CTX_SIZE * j);
				}
				for (uint32_t j = 0; j < 65536; j++)
				{
					encryptAes128(ctx, pwKey);
					for (uint32_t k = 1; k < pwBlocks; k++)
					{
						encryptAes128(ctx + AES128_CTX_SIZE * k, pwKey);
					}
				}
			}
			memcpy(pwKeys, pwKey, sizeof(pwKey));
			pwKeys += sizeof(pwKey) / sizeof(uint32_t);
		}

		// Get next work unit
		outQueue->push(node);
		node = inQueue->pop();
	}
	m_run = 0;
	return NULL;
}
