/*
	MegaCracker - Cracks the password hash from a confirmation link.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdlib.h>
#include <string.h>
#include "reader.h"

Reader::Reader(
	MessageQueue<msgQueueNodeType> *inQueue,
	MessageQueue<msgQueueNodeType> *outQueue,
	FILE                           *fin)
	: m_inQueue(inQueue),
	  m_outQueue(outQueue),
	  m_fin(fin),
	  m_read(0),
	  m_run(1)
{
	if (inQueue == NULL || outQueue == NULL || fin == NULL)
	{
		reportError("Reader::Reader(): NULL pointer");
		exit(1);
	}
}

Reader::~Reader()
{
	if (!stopRunning())
	{
		waitForStopRunning();
	}
}

int Reader::stopRunning()
{
	if (m_run)
	{
		m_run = 0;
		m_inQueue->flush();
		return 0;
	}
	return 1;
}

void *Reader::threadFunc()
{
	char                                pw[PW_BUFFER_SIZE];
	MessageQueue<msgQueueNodeType>     *inQueue  = m_inQueue;
	MessageQueue<msgQueueNodeType>     *outQueue = m_outQueue;
	messageQueueNode<msgQueueNodeType> *node;
	FILE                               *fin = m_fin;
	size_t                              pwLen;

	// Get next password
	do
	{
		if (fgets(pw, PW_BUFFER_SIZE, fin) == NULL)
		{
			if (!feof(fin))
			{
				perror("fgets");
			}
			fclose(fin);
			fin = NULL;
		}
		else
		{
			pwLen = strlen(pw);
			while (pw[pwLen - 1] == '\r' || pw[pwLen - 1] == '\n')
			{
				pwLen--;
			}
		}
		m_read = 1;
	} while (fin != NULL && pwLen > NODE_PW_BUFFER_SIZE - 2);

	// Get work unit
	node = inQueue->pop();
	while (node != NULL && fin != NULL && m_run)
	{
		uint8_t  *passwords       = (uint8_t*) node->data.passwords;
		uint32_t count            = 0;
		uint32_t pwBufferSizeLeft = NODE_PW_BUFFER_SIZE;

		while (count < NODE_MAX_PWS && pwBufferSizeLeft >= pwLen + 2)
		{
			// Copy password
			count++;
			*((uint16_t*) passwords) = (uint16_t) pwLen;
			memcpy(passwords + 2, pw, pwLen);
			passwords        += pwLen + 2;
			pwBufferSizeLeft -= pwLen + 2;

			// Get next password
			do
			{
				if (fgets(pw, PW_BUFFER_SIZE, fin) == NULL)
				{
					if (!feof(fin))
					{
						perror("fgets");
					}
					fclose(fin);
					fin = NULL;
					break;
				}
				pwLen = strlen(pw);
				while (pwLen > 0 && (pw[pwLen - 1] == '\r' || pw[pwLen - 1] == '\n'))
				{
					pwLen--;
				}
			} while (fin != NULL && pwLen > NODE_PW_BUFFER_SIZE - 2);
			if (fin == NULL)
			{
				break;
			}
		}
		node->data.count = count;

		// Get next work unit
		outQueue->push(node);
		node = inQueue->pop();
	}
	m_run = 0;
	inQueue->flush();
	outQueue->flush();
	return NULL;
}
